


alt.equidistribution.test <- function(x)
{
    ks.test(x, punif)$p.value
}

alt.gap.test <- function(x, alpha = 0.3, beta = 0.7, t,
                         verbose = FALSE, ...)
    ## find gaps between occurrences of numbers in (alpha, beta)
{
    stopifnot(alpha < beta)
    p <- (beta - alpha)
    in.int <- which(alpha < x & x < beta)
    d <- diff(in.int) - 1
    n <- length(d) ## number of gaps
    if (missing(t)) t <- floor((log(5) - log(n * p)) / log(1-p))
    t <- as.integer(t)
    d[d >= t] <- t
    pd <- p * (1-p)^(0:t)
    pd[t + 1] <- 1 - sum(head(pd, t))
    tab <- table(factor(d, levels = 0:t))
    if (verbose) {
        fdist <- cbind(counts = tab, prob = pd)
        print(rbind(fdist, total = apply(fdist, 2, sum)))
    }
    chisq.test(tab, p = pd)$p.value
}

alt.permutation.test <- function(x, t = 3, ...)
{
    base <- max(10, t + 1)
    ft <- factorial(t)
    x <- head(x, t * (length(x) %/% t))
    m <- matrix(x, nrow = t, byrow = FALSE)
    om <- apply(m, 2, order)
    num <- rep(0, ncol(om))
    for (i in seq(length = t))
    {
        num <- num + om[i, ] * base^(i-1)
    }
    tab <- table(num)
    if (length(tab) < ft) # some missing
    {
        ## fill in zeros. order doesn't matter
        tab <- c(tab, rep(0, ft - length(tab)))
    }
    chisq.test(tab, p = rep(1/ft, ft))$p.value
}

alt.maxt.test <- function(x, t = 10, ...)
{
    x <- head(x, t * (length(x) %/% t))
    m <- matrix(x, nrow = t, byrow = FALSE)
    maxt <- rep(0, ncol(m))
    for (i in seq(length = t))
    {
        maxt <- pmax(maxt, m[i, ])
    }
    ks.test(maxt, function(x) x^t)$p.value
}

alt.birthday.test <- function(x, ...)
{
    m <- as.integer(2^25)
    n <- 512L
    x <- head(as.integer(x * m),
              n * (length(x) %/% n))
    xmat <- matrix(x, nrow = n, byrow = FALSE)
    countSpacing <- function(y)
    {
        y <- sort(y)
        s <- c(diff(y),
               y[1] - y[length(y)] + m)
        s <- sort(s)
        sum(diff(s) == 0)
    }
    cs <- apply(xmat, 2, countSpacing)
    cs[cs > 3] <- 3
    tab <- table(factor(cs, levels = 0:3))
    chisq.test(tab,
               p = c(0.368801577, 0.369035243,
                     0.183471182, 0.078691997))$p.value
}

alt.all <- function(x, ...)
{
    c(gap = alt.gap.test(x, ...),
      permutation = alt.permutation.test(x, ...),
      maxt = alt.maxt.test(x, ...),
      birthday = alt.birthday.test(x, ...))
}


